/**********************************************************************
*	FILE:	viewport.h
*	AUTHOR:	CALeague
*	DATE:	25 April 1993
*	PURPOSE:
*	  A Viewport is a rectangular portion of the physical screen in
*	  which one can draw pixel-oriented objects such as circles
*	  and lines.  Viewports support remember information about
*	  their colors, text-styles, line-styles, etc., and automatically
*	  switch to these attributes if necessary.
***********************************************************************/

#ifndef _VIEWPORT_H
#define _VIEWPORT_H

#include <stddef.h>
#include "typedefs.h"
#include "video.h"
#include "msmouse.h"


/**********************************************************************
*	Settings structures
***********************************************************************/

struct ViewSettings
{
  int X1, Y1, X2, Y2;
  ViewSettings (int x1, int y1, int x2, int y2)
    { X1 = x1;  Y1 = y1;  X2 = x2;  Y2 = y2; }
  ViewSettings (void)
    { X1 = Y1 = 0;  X2 = Screen.GetMaxX ();  Y2 = Screen.GetMaxY (); }
  void Install (void)
    { setviewport (X1, Y1, X2, Y2, True); }
};


struct ColorSettings
{
  int BG, FG;
  ColorSettings (void)
    { BG = BLACK;  FG = WHITE; }
  void Install (void)
    { setcolor (FG); }
};


struct FillSettings
{
  int  Pattern, Color;
  FillSettings (void)
    { Pattern = SOLID_FILL;  Color = WHITE; }
  void Install (void)
    { setfillstyle (Pattern, Color); }
};


struct LineSettings
{
  int Style, Thickness;
  unsigned Pattern;
  LineSettings (void)
    { Style = SOLID_LINE;  Pattern = 0x01; Thickness = NORM_WIDTH; }
  void Install (void)
    { setlinestyle (Style, Pattern, Thickness); }
};


struct TextSettings
{
  int Font, Direction, CharSize, HJust, VJust;
  TextSettings (void)
    { Font = DEFAULT_FONT;  Direction = HORIZ_DIR;  CharSize = 1;
      HJust = LEFT_TEXT;  VJust = TOP_TEXT; }
  void Install (void)
    { settextstyle (Font, Direction, CharSize);
      settextjustify (HJust, VJust); }
};


struct Position
{
  int X, Y;
  Position (void) { X = Y = 0; }
  void Install (void)
    {  moveto (X, Y); }
  void Save (void)
    { X = getx ();  Y = gety (); }
};


/**********************************************************************
*	Class Viewport
***********************************************************************/

class Viewport
{                               // Private settings data
protected:
  ViewSettings  SView;
  ColorSettings SColor;
  FillSettings  SFill;
  LineSettings  SLine;
  TextSettings  SText;
  Position      SPos;
  Viewport*	Parent;
				// Context management stuff
  static Viewport* Context;
  void Install (void);
  void SwitchTo (void) { if (Context != this) Install (); }

				// In preparation for drawing:
  void DoDraw (void)   { if (!Fast) Mouse.Hide (); }
  void DoneDraw (void) { if (!Fast) Mouse.Show (); }
  static int Fast;
public:
  void FastDraw (void) { if (!Fast) Mouse.Hide (); Fast++; }
  void Done (void)     { Fast--; if (!Fast) Mouse.Show (); }

				// Constructors
  Viewport (Viewport* parent, int x1, int y1, int x2, int y2);
  Viewport (void);

				// Modifiers

  void SetColor   (int c) 	{ SColor.FG = c;  Context = NULL; }
  void SetBkColor (int c) 	{ SColor.BG = c;  Context = NULL; }
  void SetFillStyle (int pat, int col)
    { SFill.Pattern = pat;  SFill.Color = col;  Context = NULL; }
  void SetLineStyle (int st, unsigned pat, int thick)
    { SLine.Style = st;  SLine.Pattern = pat;  SLine.Thickness = thick;
      Context = NULL; }
  void SetTextStyle (int fnt, int dir, int size)
    { SText.Font = fnt;  SText.Direction = dir;  SText.CharSize = size;
      Context = NULL; }
  void SetTextJustify (int hor, int ver)
    { SText.HJust = hor;  SText.VJust = ver;
      Context = NULL; }

				// Selector Functions

  int TextHeight (char* s)
    { SwitchTo ();  return textheight (s); }
  int TextWidth (char* s)
    { SwitchTo ();  return textwidth (s);  }
  int GetBkColor (void)  { return SColor.BG; }
  int GetColor (void)	 { return SColor.FG; }
  int GetMaxColor (void) { return Screen.GetMaxColor (); }
  int GetMaxX (void)	 { return (SView.X2 - SView.X1); }
  int GetMaxY (void)	 { return (SView.Y2 - SView.Y1); }
  int GetX (void)	 { return SPos.X; }
  int GetY (void)	 { return SPos.Y; }
  int TransX (int x);		// Translate parent's coordinate system
  int TransY (int y);		//  to ours.
  bool InBounds (int x, int y);	// Check whether (x,y) is in this viewport

				// Drawing Functions

  void Bar (int x1, int y1, int x2, int y2)
    { SwitchTo (); DoDraw (); bar (x1, y1, x2, y2); DoneDraw ();}
  void Circle (int x, int y, int r)
    { SwitchTo (); DoDraw (); circle (x, y, r); DoneDraw ();}
  void Clear ();
  void FillEllipse (int x, int y, int xr, int yr)
    { SwitchTo (); DoDraw (); fillellipse (x, y, xr, yr); DoneDraw ();}
  void FloodFill (int x, int y, int border)
    { SwitchTo (); DoDraw (); floodfill (x, y, border); DoneDraw ();}
  void GetImage (int x1, int y1, int x2, int y2, void far *img)
    { SwitchTo (); getimage (x1, y1, x2, y2, img); }
  unsigned GetPixel (int x, int y)
    { SwitchTo (); return getpixel (x, y); }
  void Line (int x1, int y1, int x2, int y2)
    { SwitchTo (); DoDraw (); line (x1, y1, x2, y2); DoneDraw ();}
  void LineRel (int dx, int dy)
    { SwitchTo (); DoDraw (); linerel (dx, dy); DoneDraw (); SPos.Save (); }
  void LineTo (int x, int y)
    { SwitchTo (); DoDraw (); lineto (x, y); DoneDraw (); SPos.Save (); }
  void MoveRel (int dx, int dy)
    { SwitchTo (); moverel (dx, dy);  SPos.Save (); }
  void MoveTo (int x, int y)
    { SwitchTo (); moveto (x, y);  SPos.Save (); }
  void OutText (char* s)
    { SwitchTo (); DoDraw (); outtext (s); DoneDraw ();}
  void OutTextXY (int x, int y, char* s)
    { SwitchTo (); DoDraw (); outtextxy (x, y, s); DoneDraw ();}
  void PutImage (int x, int y, void far *img, int op)
    { SwitchTo (); DoDraw (); putimage (x, y, img, op); DoneDraw ();}
  void PutPixel (int x, int y, int clr)
    { SwitchTo (); DoDraw (); putpixel (x, y, clr); DoneDraw ();}
  void Rectangle (int x1, int y1, int x2, int y2)
    { SwitchTo (); DoDraw (); rectangle (x1, y1, x2, y2); DoneDraw ();}

};


#endif

